@@ -1,10 +1,15 @@
 #  Copyright (c) 2022 by Amplo.
 
 from amplo.automl.data_processing import DataProcessor
-from amplo.automl.drift_detection import DriftDetector
-from amplo.automl.feature_processing import FeatureProcessor
+from amplo.automl.feature_processing.feature_processing import FeatureProcessor
 from amplo.automl.grid_search import OptunaGridSearch
-from amplo.automl.interval_analysis import IntervalAnalyser
 from amplo.automl.modelling import Modeller
-from amplo.automl.sequencing import Sequencer
 from amplo.automl.standardization import Standardizer
+
+__all__ = [
+    "DataProcessor",
+    "FeatureProcessor",
+    "OptunaGridSearch",
+    "Modeller",
+    "Standardizer",
+]
