@@ -6,3 +6,12 @@ from amplo.classification.lgbm import LGBMClassifier
 from amplo.classification.partial_boosting import PartialBoostingClassifier
 from amplo.classification.stacking import StackingClassifier
 from amplo.classification.xgb import XGBClassifier
+
+__all__ = [
+    "BaseClassifier",
+    "CatBoostClassifier",
+    "LGBMClassifier",
+    "PartialBoostingClassifier",
+    "StackingClassifier",
+    "XGBClassifier",
+]
