@@ -5,44 +5,29 @@ from __future__ import annotations
 import json
 import os
 import re
+from logging import Logger
 from pathlib import Path
 from time import time
-from typing import TYPE_CHECKING, Iterable
+from typing import Any, Iterable
 from warnings import warn
 
 import numpy as np
 import pandas as pd
 from requests import HTTPError
 
+from amplo.api.platform import AmploPlatformAPI
 from amplo.api.storage import AzureBlobDataAPI
 from amplo.utils.logging import get_root_logger
 
-if TYPE_CHECKING:
-    from logging import Logger
-
-    from amplo.api.platform import AmploPlatformAPI
-
 __all__ = [
     "boolean_input",
     "parse_json",
     "NpEncoder",
-    "read_pandas",
     "get_file_metadata",
     "merge_logs",
 ]
 
 
-FILE_READERS = {
-    ".csv": pd.read_csv,
-    ".json": pd.read_json,
-    ".xml": pd.read_xml,
-    ".feather": pd.read_feather,
-    ".parquet": pd.read_parquet,
-    ".stata": pd.read_stata,
-    ".pickle": pd.read_pickle,
-}
-
-
 class NpEncoder(json.JSONEncoder):
     def default(self, obj):
         if isinstance(obj, np.integer):
@@ -55,7 +40,7 @@ class NpEncoder(json.JSONEncoder):
             return obj.to_list()
         if isinstance(obj, pd.DataFrame):
             obj.to_json()
-        return super(NpEncoder, self).default(obj)
+        return super().default(obj)
 
 
 def boolean_input(question: str) -> bool:
@@ -69,7 +54,7 @@ def boolean_input(question: str) -> bool:
         return boolean_input(question)
 
 
-def parse_json(json_string: str | dict) -> str | dict:
+def parse_json(json_string: str | dict[Any, Any]) -> str | dict[Any, Any]:
     if isinstance(json_string, dict):
         return json_string
     else:
@@ -86,22 +71,6 @@ def parse_json(json_string: str | dict) -> str | dict:
             return json_string
 
 
-def read_pandas(path: str | Path) -> pd.DataFrame:
-    """
-    Wrapper for various read functions
-
-    Returns
-    -------
-    pd.DataFrame
-    """
-    file_extension = Path(path).suffix
-    if file_extension not in FILE_READERS:
-        raise NotImplementedError(f"File format {file_extension} not supported.")
-    else:
-        reader = FILE_READERS[file_extension]
-        return reader(path, low_memory=False)
-
-
 def get_file_metadata(file_path: str | Path) -> dict[str, str | float]:
     """
     Get file metadata from given path.
@@ -126,7 +95,7 @@ def get_file_metadata(file_path: str | Path) -> dict[str, str | float]:
 
     from amplo.utils import check_dtypes
 
-    check_dtypes("file_path", file_path, (str, Path))
+    check_dtypes(("file_path", file_path, (str, Path)))
 
     file_path = Path(file_path)
 
@@ -155,8 +124,9 @@ def _get_api_clients(
     if not azure:
         blob_api = None
     else:
-        azure = azure if not isinstance(azure, bool) else tuple()
-        blob_api = AzureBlobDataAPI.from_os_env(*azure)
+        blob_api = AzureBlobDataAPI.from_os_env(
+            *azure if isinstance(azure, tuple) else (None, None)
+        )
 
     # Mirror azure parameter when platform is not set
     if platform is None:
@@ -165,8 +135,9 @@ def _get_api_clients(
     if not platform:
         platform_api = None
     else:
-        platform = platform if not isinstance(platform, bool) else tuple()
-        platform_api = AmploPlatformAPI.from_os_env(*platform)
+        platform_api = AmploPlatformAPI.from_os_env(
+            *platform if isinstance(platform, tuple) else (None, None)
+        )
     return blob_api, platform_api
 
 
@@ -178,21 +149,22 @@ def _get_folders(
     """
     Lists folders for merge_logs
     """
-    if not blob_api:
+    folders: list[Path]
+    if blob_api:
+        folders = [Path(f) for f in blob_api.ls_folders(parent_folder)]
+    else:
         if not Path(parent_folder).exists():
             raise ValueError(f"{parent_folder} directory does not exist.")
         folders = [
             folder for folder in Path(parent_folder).iterdir() if folder.is_dir()
         ]
-    else:
-        folders = blob_api.ls_folders(parent_folder)
 
     # Add more_folders
     if more_folders:
-        folders += more_folders
+        folders += [Path(f) for f in more_folders]
 
     # return [Path(f) for f in folders]
-    return [Path(f) for i, f in enumerate(folders) if i < 3]
+    return [Path(f) for f in folders]
 
 
 def _read_files_in_folders(
@@ -200,7 +172,7 @@ def _read_files_in_folders(
     target: str,
     blob_api: AzureBlobDataAPI | None = None,
     logger: Logger | None = None,
-) -> tuple[list[str], pd.DataFrame, dict[str, dict]]:
+) -> tuple[list[str], pd.DataFrame, dict[str, dict[str, Any]]]:
     """
     Use pandas to read all non-hidden and non-empty files into a DataFrame.
 
@@ -229,16 +201,16 @@ def _read_files_in_folders(
     """
 
     # Map folders to pathlib.Path object
-    folders = [Path(f) for f in folders]
+    folder_paths = [Path(f) for f in folders]
 
     # Initialize
     file_names, data, metadata = [], [], {}
     last_time_logged = time()
-    for folder_count, folder in enumerate(sorted(folders)):
+    for folder_count, folder in enumerate(sorted(folder_paths)):
 
         # List all files
         if blob_api:
-            files = list(map(Path, blob_api.ls_files(folder)))
+            files = [Path(f) for f in blob_api.ls_files(folder)]
         else:
             files = [f for f in folder.iterdir() if f.is_file()]
 
@@ -246,10 +218,6 @@ def _read_files_in_folders(
         hidden_files = [f for f in files if re.match(r"^\..*", f.name)]
         files = list(set(files) - set(hidden_files))
 
-        # Remove unsupported file types
-        unsupported_files = [f for f in files if f.suffix not in FILE_READERS]
-        files = list(set(files) - set(unsupported_files))
-
         # Remove empty files
         if blob_api:
             empty_files = [f for f in files if blob_api.get_size(f) == 0]
@@ -263,22 +231,26 @@ def _read_files_in_folders(
             continue
 
         # Read files
-        for file_ in sorted(files):
+        for i, file_ in enumerate(sorted(files)):
 
             # read_pandas() may raise an EmptyDataError when the file has no content.
             # The try...except catches such errors and warns the user instead.
             try:
                 if blob_api:
-                    datum = blob_api.read_pandas(file_)
+                    datum = blob_api.read_pandas(file_, low_memory=False)
                     metadatum = blob_api.get_metadata(file_)
                 else:
-                    datum = read_pandas(file_)
+                    datum = pd.read_parquet(file_)
                     metadatum = get_file_metadata(file_)
             except pd.errors.EmptyDataError:
                 warn(f"Empty file detected and thus skipped: '{file_}'")
                 continue
 
             # Convert to dataframe
+            if logger:
+                logger.debug(
+                    f"{file_} {len(datum)} samples, {len(datum.keys())} columns."
+                )
             if isinstance(datum, pd.Series):
                 datum = datum.to_frame()
 
@@ -299,25 +271,25 @@ def _read_files_in_folders(
 
         if logger and time() - last_time_logged > 90:
             last_time_logged = time()
-            logger.info(f".. progress: {folder_count / len(folders) * 100:.1f} %")
+            logger.info(f".. progress: {folder_count / len(folder_paths) * 100:.1f} %")
 
     # Concatenate data
-    data = pd.concat(data, axis=0)
+    df = pd.concat(data, axis=0)
 
     # Validate data
-    if target not in data:
+    if target not in df:
         raise ValueError("Target not in data.")
-    if data[target].nunique() != 2:
-        raise ValueError(f"Number of unique labels is {data[target].nunique()} != 2.")
+    if df[target].nunique() != 2:
+        raise ValueError(f"Number of unique labels is {df[target].nunique()} != 2.")
 
-    return file_names, data, metadata
+    return file_names, df, metadata
 
 
 def _map_datalogs_to_file_names(
     file_names: list[str],
     platform_api: AmploPlatformAPI | None = None,
     logger: Logger | None = None,
-) -> dict:
+) -> dict[str, Any]:
     """
     Get datalogs for every filename.
 
@@ -345,11 +317,12 @@ def _map_datalogs_to_file_names(
 
     # Remove path prefixes, otherwise datalogs will not be found
     file_names = ["/".join(str(fname).split("/")[-6:]) for fname in file_names]
+
     # Extract team and machine
     try:
         team, machine = file_names[0].split("/")[-6:-4]
     except IndexError:
-        warn(f"Got an empty list of file names")
+        warn("Got an empty list of file names")
         return {}
 
     # Get datalog for each filename
@@ -371,7 +344,7 @@ def _map_datalogs_to_file_names(
     return datalogs
 
 
-def _mask_intervals(datalogs: dict, data: pd.DataFrame) -> pd.DataFrame:
+def _mask_intervals(datalogs: dict[str, Any], data: pd.DataFrame) -> pd.DataFrame:
     """
     Masks the data with the intervals given by the datalogs.
 
@@ -409,7 +382,7 @@ def _mask_intervals(datalogs: dict, data: pd.DataFrame) -> pd.DataFrame:
         # Convert ts_col
         if not pd.api.types.is_numeric_dtype(data[ts_col]):
             data[ts_col] = (
-                pd.to_datetime(data[ts_col], errors="coerce").astype(int) / 10**9
+                pd.to_datetime(data[ts_col], errors="coerce").view(int) / 10**9
             )
 
         # Extract intervals
@@ -419,9 +392,9 @@ def _mask_intervals(datalogs: dict, data: pd.DataFrame) -> pd.DataFrame:
             drop_mask = (
                 (data[ts_col] < ts_first) | (data[ts_col] > ts_last)
             ) & drop_mask
-        if not drop_mask.loc[filename].any():
+        if isinstance(drop_mask, pd.Series) and not drop_mask.loc[filename].any():
             continue
-        data = data.drop(data.loc[(filename, drop_mask), :].index)
+        data.drop(data.loc[(filename, drop_mask), :].index, inplace=True)
 
     return data
 
@@ -432,7 +405,7 @@ def merge_logs(
     more_folders: list[str | Path] | None = None,
     azure: tuple[str, str] | bool = False,
     platform: tuple[str, str] | bool | None = None,
-) -> tuple[pd.DataFrame, dict[str, dict]]:
+) -> tuple[pd.DataFrame, dict[str, dict[str, Any]]]:
     """
     Combine log files of all subdirectories into a multi-indexed DataFrame.
 
@@ -499,7 +472,7 @@ def merge_logs(
     blob_api, platform_api = _get_api_clients(azure, platform)
 
     # Get child folders
-    folders = _get_folders(parent, blob_api)
+    folders = _get_folders(parent, blob_api, more_folders)
     if target not in [f.name for f in folders]:
         raise ValueError(f"Target {target} not present in folders.")
     logger.info(f"Found {len(folders)} folders.")
